/*-----------------------------------------------------*/
/*--	Micro-EMPIX UART Drivers (Low)(UARTLow.c)     -*/
/*--    Ver 1.0 (c) 2004                              -*/
/*--	E             -*/
/*--    ==================================            -*/
/*--    .                  -*/
/*--    .                        -*/
/*--    ..                       -*/
/*-----------------------------------------------------*/

#ifndef _DEFS_H
	#define _DEFS_H
	#include "defs.h"
#endif

#ifndef _ASM_H
	#define _ASM_H
	#include "asm.h"
#endif

/* ====================================================*/
/* ============== Function Prototypes =================*/
/* ====================================================*/
void UnSetControlBit(int baseport);
void SetControlBit(int baseport);
void SetBaudRate(int baseport, BAUD_RATE brate);
void SetWordLength(int baseport);
void ClearLSR_MSR_IIR(int baseport);
void ClearAndEnableFIFO(int baseport, unsigned char chh);
void NrOfCharsBefReadIRQ(int baseport, unsigned char chh);
void IER_Enable(int baseport, unsigned char IR_Byte);
void MCR_Enable(int baseport);
void IER_Disable(int baseport);
void interrupt InterruptHandlerForCOM1();
void interrupt InterruptHandlerForCOM2();
void SER_InitializePort(SER_PORT portnum, BAUD_RATE brate, const unsigned char HWDFlowControl, struct IrqConf *actions);
int SER_WriteChar(SER_PORT portnum, char WriteCh);
int SER_ReadChar(SER_PORT portnum, char *ReadCh);
int InterruptReason(SER_PORT port);




int SER_baseport1;
int SER_baseport2;

/* For Serial Port Hardware Flow Control */
unsigned char HardwareFlowControl1;
unsigned char HardwareFlowControl2;

unsigned int CharsSent1;
unsigned int CharsSent2;
unsigned char PrevSignal1;
unsigned char PrevSignal2;

/* For Interrupt Driven Serial Ports */
/*   interrupt handler  COM-1,      */
void interrupt (*IrqHandler_For_Com1)();
void interrupt (*IrqHandler_For_Com2)();
unsigned char IrqHandlerForCom1_Changed;
unsigned char IrqHandlerForCom2_Changed;

/*        */
/*    Data segment       */
/*   interrupt handler,       */
/*  global variables.  data segment    */
/*   interrupt handler   28 (DOS IDLE LOOP).  */
/*          interrupt handler */
/* (OldInt28) */
unsigned int OldInt28;

void (*SER1_func)(void);
void (*SER2_func)(void);

/*   Cbit = 0,  LCR */
void UnSetControlBit(int baseport)
{
        asm MOV DX,baseport
        asm ADD DX,3
        asm IN AL,DX
        asm AND AL,7FH
        asm OUT DX,AL
}

/*   Cbit = 1,  LCR */
void SetControlBit(int baseport)
{
        asm MOV DX,baseport
        asm ADD DX,3
        asm IN AL,DX
        asm OR AL,80H
        asm OUT DX,AL
}

/*   Baud Rate */
void SetBaudRate(int baseport, BAUD_RATE brate)
{
    /* Setting BaudRate */
    /* Set line control 7th bit to 1, so that we can talk to DLL and DLM */
    SetControlBit(baseport);
	/* Computing Baud rate for DLL */
	asm MOV BX,brate
	asm AND BX,0FFH
    /* Talk to DLL (Baud rate divisor least significant byte) */
	asm MOV DX,baseport;
	asm MOV AL,BL
	asm OUT DX,AL
	/* Computing Baud rate for DLM */
	asm MOV BX,brate
	asm AND BX,0FF00H
	/* Talk to DLM (Baud rate divisor most significant byte) */
    asm MOV DX,baseport;
	asm INC DX
	asm MOV AL,BH
	asm OUT DX,AL
    /* Make 7th bit of LCR is 0 */
    UnSetControlBit(baseport);
}

/*   LCR, o    (8 bits), 1 stop bit    */
void SetWordLength(int baseport)
{
	/* Setting Word length, Stop bits and Parity */
    port_out(baseport, 3, 0x3); /* 8 bits word length, 1 stop bit and no parity */
}


/*   LSR, MSR & IIR     interrupts */
void ClearLSR_MSR_IIR(int baseport)
{
        /* Clearing LSR, MSR, IIR */
        asm MOV DX,baseport
        asm ADD DX,5
        asm IN AL,DX
        asm INC DX
        asm IN AL,DX
        asm SUB DX,4
        asm IN AL,DX
}

/*         16550,  */
/*  FIFO buffers        */
/* (    ) */
void ClearAndEnableFIFO(int baseport, unsigned char chh)
{
        /* Clear internal 16550 buffers and enable FIFO buffers */
        /* Enable new buffers */
        port_out(baseport, 2, chh | 3);
}

/*        interrupts   16550,   */
/*          16550,     */
/* interrupt.     1, 4, 8  12.      FIFO Control register */
void NrOfCharsBefReadIRQ(int baseport, unsigned char chh)
{
        /* Set number of chars before each interrupt */
        port_out(baseport, 2, chh | 3);
}

/*          interrupt   UART,  IER */
void IER_Enable(int baseport, unsigned char IR_Byte)
{
        /* Insane measure. Make sure that 7th bit of LCR is 0 */
        UnSetControlBit(baseport);
        /* Enable interrupts in IER */
        port_out(baseport, 1, IR_Byte);
}

/*     MCR */
void MCR_Enable(int baseport)
{
        /* Enable interrupts in Modem control register */
        port_out(baseport, 4, 0x0b);
}

/*        interrupts  IER */
void IER_Disable(int baseport)
{
        /* Disable interrupts in IER */
        port_out(baseport, 1, 0);
}

/*          ,   interrupt hanlder */
/*    .              SER_func, */
/*                struct actions */
/*     SER_InitializePort */

void interrupt InterruptHandlerForCOM1()
{

    /* Restore the original DS */
	asm XOR AX,AX
	asm MOV ES,AX
	asm MOV BX,ds_store
	asm MOV DS,WORD PTR ES:[BX]

	asm XOR AX,AX
	asm PUSH AX

	SER1_func();

	asm ADD SP,2

	/* Tell the 8259 PIC chip that we have served the interrupt */
	asm MOV AL,20H
	asm OUT 20H,AL

}

/*      */

void interrupt InterruptHandlerForCOM2()
{

    /* Restore the original DS */
	asm XOR AX,AX
	asm MOV ES,AX
	asm MOV BX,ds_store
	asm MOV DS,WORD PTR ES:[BX]

	asm XOR AX,AX
	asm PUSH AX

        SER2_func();

	asm ADD SP,2

	/* Tell the 8259 PIC chip that we have served the interrupt */
	asm MOV AL,20H
	asm OUT 20H,AL

}

/*        portnum.       */
/*      SER.  brate   Baud Rate (   enum),*/
/*  HWDFlowControl     0,    actions   NULL,    */
/*     polling.          interrupts */
/*            actions */
void SER_InitializePort(SER_PORT portnum, BAUD_RATE brate, const unsigned char HWDFlowControl, struct IrqConf *actions)
{
	int baseport;
	unsigned char chh;

	if(portnum != SER_P1 && portnum != SER_P2) return;
    if(portnum == SER_P1 && SER_baseport1) /* Serial already initialized */
		return;
    if(portnum == SER_P2 && SER_baseport2) /* Serial already initialized */
        return;
    asm PUSHF
    asm CLI

    /* The following must be done only the first time */
    if(!SER_baseport1 && !SER_baseport2)
    {
		/* Save the contents of the interrupt handler */
        asm XOR AX,AX
        asm MOV ES,AX
        asm MOV BX,ds_store
        asm MOV AX, WORD PTR ES:[BX]
        asm MOV WORD PTR OldInt28, AX
        /* Insert our current DS */
        asm XOR AX,AX
        asm MOV ES,AX
        asm MOV AX,DS
        asm MOV BX,ds_store
        asm MOV WORD PTR ES:[BX], AX
    }

    /* Get Port Num from Bios */
    asm MOV BX,40H
    asm MOV ES,BX
    asm XOR BX,BX
    asm ADD BX,portnum
    asm MOV AX, WORD PTR ES:[BX]
    asm MOV baseport,AX

    if(portnum == SER_P1)
    {
		SER_baseport1 = baseport;
        HardwareFlowControl1 = HWDFlowControl;
    }
    else if(portnum == SER_P2)
    {
       HardwareFlowControl2 = HWDFlowControl;
	   SER_baseport2 = baseport;
    }

    SetBaudRate(baseport, brate);
    SetWordLength(baseport);

    ClearLSR_MSR_IIR(baseport);

    /* Insane measure. Make sure that 7th bit of LCR is 0 */
    UnSetControlBit(baseport);
    /* Read and clear any left over characters */
    asm MOV DX,baseport
LOOP:
	asm ADD DX,5
    /* Get line status */
	asm IN AL,DX
    /* Check if char available */
	asm TEST AL,01H
    asm JZ CHARNOTFOUND
    asm SUB DX,5
	asm IN AL,DX
    asm JMP LOOP  /* Are all available characters read? */
CHARNOTFOUND:
    ClearAndEnableFIFO(baseport, actions->Chars_Bef_Irq);
	/* Deal with interrupts */
	if(actions)
	{
		/* Set interrupt handler */
		if(portnum == SER_P1)
		{
			IrqHandlerForCom1_Changed = 1;
			SER1_func = actions->IrqActions;
            IrqHandler_For_Com1 = (void interrupt (*)())(getvec(IRQVEC_FOR_SER_P1));
			setvec(IRQVEC_FOR_SER_P1, InterruptHandlerForCOM1);
			/* Disable flag 4 in 8259 */
			asm MOV DX,21H
			asm IN AL,DX
			asm AND AL,0EFH
			asm OUT DX,AL
		}
		else
		{
			IrqHandlerForCom2_Changed = 1;
            SER2_func = actions->IrqActions;
            IrqHandler_For_Com2 = (void interrupt (*)())(getvec(IRQVEC_FOR_SER_P2));
            setvec(IRQVEC_FOR_SER_P2, InterruptHandlerForCOM2);
			/* Disable flag 3 in 8259 */
			asm MOV DX,21H
			asm IN AL,DX
			asm AND AL,0F7H
			asm OUT DX,AL
		}

		/* NrOfCharsBefReadIRQ(baseport, actions->Chars_Bef_Irq);*/
		IER_Enable(baseport, actions->IR_Byte);
		MCR_Enable(baseport);
	}
	else
	{
		if(portnum == SER_P1)
			IrqHandlerForCom1_Changed = 0;
		else if(portnum == SER_P2)
			IrqHandlerForCom2_Changed = 0;

        IER_Disable(baseport);
	}

    if(HWDFlowControl)
    {
        /* Setting RTS to 0*/
        asm MOV DX,baseport
        asm ADD DX,4
        asm IN AL,DX
        asm AND AL,0FDH
        asm OUT DX,AL
        /* Make 0 the counters */
        CharsSent1 = MAXCHARS_BEFORE_STOPPING;
        CharsSent2 = MAXCHARS_BEFORE_STOPPING;
        PrevSignal1 = 0;
        PrevSignal2 = 0;
     }
     asm POPF
     /*     THR_Empty interrupts,      */
	 /*    ,  UART      interrupt */
	 /*       (   )  */
	 /*             . */
	 /*        interrupt  ,   */
	 /*      UART,       */
	 /*  interrupts.          IIR,*/
	 /*  acknowledge sto UART. */

     InterruptReason(portnum);
}

/* H        polling     */
/* interrupt handlers.         */
/*   UART      ReadCh,        */
/* 16 .       .      */
/*  0.      -2,       */
/*  buffers  UART       (overwrite error)  -1 */

int SER_ReadChar(SER_PORT portnum, char *ReadCh)
{
        int baseport;
        char ans;
        int mret = 0;
        char *ptr = ReadCh;
        unsigned char HardwareFlowControl;

        if(portnum == SER_P1)
        {
                HardwareFlowControl = HardwareFlowControl1;
				baseport = SER_baseport1;
        }
        else if(portnum == SER_P2)
        {
                HardwareFlowControl = HardwareFlowControl2;
				baseport = SER_baseport2;
        }
        else
                return -10; /* Not valid serial port */

        if(HardwareFlowControl)
        {
                /* Characters read. Send signal to disable sending of more chars*/
                asm MOV DX,baseport
                asm ADD DX,4
                asm IN AL,DX
                asm AND AL,0FDH /* make RTS 0 */
                asm OUT DX,AL
        }
LOOP:
        /* Check the corresponding bit of the control status register */
		asm MOV DX,baseport
		asm ADD DX,5
        /* Get line status */
		asm IN AL,DX
        asm TEST AL,08H
		asm JNZ TR_ERROR
		asm TEST AL,02H
		asm JNZ CHAR_OVERFLOW
		asm TEST AL,01H
        asm JZ CHARNOTFOUND
        asm SUB DX,2
		asm IN AL,DX
		asm AND AL,7FH
		asm OUT DX,AL
		asm SUB DX,3
		asm IN AL,DX
        asm MOV ans,AL
        *ptr++ = ans;
        mret++;
        asm JMP LOOP  /* Are all available characters read? */

        /* Hardware Flow control */
        if(HardwareFlowControl && mret == 0) /* If we have hardware flow control and no characters read */
        {
                /* Checking for signal */
                asm MOV DX,baseport
                asm ADD DX,6
                asm IN AL,DX
                asm TEST AL,10H
                asm JZ CHARNOTFOUND /* An den eixame shma gia flow control apla synexizoume */
                /* Sending signal for permission */
                asm MOV DX,baseport
                asm ADD DX,4
                /* send signal for more data */
                asm IN AL,DX
                asm OR AL,02H
                asm OUT DX,AL
        }

CHARNOTFOUND:
        return mret; /* If mret == 0, No char returned */
CHAR_OVERFLOW:
        return -1; /* Character overwritten at the serial buffers */
TR_ERROR:
        return -2; /* Transmission error */
}

/*          UART.     */
/*     polling     interrupt handlers.     */
/* 1      .  -2   transmission error.   */
/* -1    buffers  UART           */
/* .   2       .    */
/*            <> 1 */

int SER_WriteChar(SER_PORT portnum, char WriteCh)
{
        int baseport, retval = 1;
        unsigned char HardwareFlowControl;
        unsigned char *PrevSignal, CurSignal;
        unsigned int *CharsSent;

        if(portnum == SER_P1)
        {
                HardwareFlowControl = HardwareFlowControl1;
                PrevSignal = &PrevSignal1;
                CharsSent = &CharsSent1;
				baseport = SER_baseport1;
        }
        else if(portnum == SER_P2)
        {
                HardwareFlowControl = HardwareFlowControl2;
                PrevSignal = &PrevSignal2;
                CharsSent = &CharsSent2;
				baseport = SER_baseport2;
        }
        else return -10; /* Not valid serial port */

        /* Hardware flow control check (if necessary) */
        if(HardwareFlowControl)
        {
                /* Checking for permission */
                asm MOV DX,baseport
                asm ADD DX,6
                asm IN AL,DX
                asm AND AL,10H
                asm MOV CurSignal,AL
                if(*PrevSignal !=  CurSignal) /* Change in state. Clear to send */
                {
                        *CharsSent = 1;
                        goto SENDCHAR;
                }
                else if(CurSignal == *PrevSignal && *CharsSent < MAXCHARS_BEFORE_STOPPING)
                {
                        (*CharsSent)++;
                        goto SENDCHAR;
                }
                else
                {
                        /* Send signal for permission */
                        asm MOV DX,baseport
                        asm ADD DX,4
                        /* intention to send data */
                        asm IN AL,DX
                        asm OR AL,02H
                        asm OUT DX,AL
                        return 3; /* sent signal and waiting for permission */
                }
        }
SENDCHAR:
		asm MOV DX,baseport
		asm ADD DX,5
		/* Get line status */
		asm IN AL,DX
		asm TEST AL,08H
		asm JNZ TR_ERROR
        /* Check THR (Transmitter holding register) */
		asm TEST AL,20H
	    asm JZ THR_NOT_EMPTY
		asm TEST AL,40H
        asm JZ TSR_NOT_EMPTY
CONT:	asm SUB DX,2
		asm IN AL,DX
		asm AND AL,7FH /* Set 7th bit of line control to 0 */
		asm OUT DX,AL
		asm SUB DX,3
		asm MOV AL,BYTE PTR WriteCh
		asm OUT DX,AL
        return retval;
TSR_NOT_EMPTY:
        return 2;
        /* data are being sent at this very moment and have not reached the recipient yet from */
        /* the previous char */
THR_NOT_EMPTY:
        return -1; /* There are data waiting to be sent in THR */
TR_ERROR:
        return -2; /* Transmission error */
}

/* H       interrupt handler       */
/*        .   IIR */
/* Returns the cause of the interrupt (used only inside interrupt handlers) */
/* Returns 0 if the specified port hasn't triggered the interrupt */
/* Returns 1 if new char is available */
/* Returns 2 if a new char can be sent (THR Empty) */
/* Returns 3 if modem status has changed */
/* Returns -1 if a transmission error occured */
int InterruptReason(SER_PORT port)
{
        int baseport;
        if(port == SER_P1)
                baseport = SER_baseport1;
        else
                baseport = SER_baseport2;
        /* Checking the serial IIR to determine what caused the interrupt */
        asm MOV DX,baseport
		asm ADD DX,2
		asm IN AL,DX
		asm TEST AL,01H
		/* If bit 0 is 1 then another port triggered the interrupt */
        asm JNZ end
cause:
		asm TEST AL,02H
		asm JNZ THR_NOT_EMPTY
		asm TEST AL,04H
		asm JNZ CHAR_READY
		asm TEST AL,06H
		asm JNZ TRANSM_ERROR
		/* diaforetika einai status change */
		return 3;

THR_NOT_EMPTY:
		return 2;
CHAR_READY:
		return 1;
TRANSM_ERROR:
		return -1;
end:
		return 0;
}


