/*-----------------------------------------------------*/
/*--	Micro-EMPIX Global Definitions    (defs.h)    -*/
/*--    Ver 1.0 (c) 2004                              -*/
/*--	E             -*/
/*--    ==================================            -*/
/*--    .                  -*/
/*--    .                        -*/
/*--    . .                      -*/
/*-----------------------------------------------------*/

/* ====================================================*/
/* ==========GLOBAL DEFINITIONS FOR OS FEATURES =======*/
/* ====================================================*/

#define QUANTUM 2               /* The time quantum given to each process */
#define MAXPROC 5 				/* The maximum number of processes allowed */
#define MAXSEM  20				/*     */
#define STACKSIZE 2000 			/* The size of the stack for each process */

#define PNMLEN 15
#define PNREGS 14

#define RUNNING 'R'
#define BLOCKED 'B'
#define READY 'A'

#define RBP         0
#define RAX         1
#define RBX         2
#define RCX         3
#define RDX         4
#define RSI         5
#define RDI         6
#define RDS         7
#define RSS         8
#define RES         9
#define RSP         10
#define RIP         11
#define RCS         12
#define PSW        13
#define INITPS     0x0200

#define OK 1
#define ERROR 0

struct pcb
{
	 char  pstate;      		/*     */
	 int pid;		   			/*  PID   */
	 int pprio;         		/*    */
	 int pregs[PNREGS]; 		/*     */
	 char pname[PNMLEN];		/*     */
	 int type;	       			/*    */
};

/* ====================================================*/
/* ==========   USEFUL MACROS        ==================*/
/* ====================================================*/

#define getoffset(a)	((unsigned)(0x0000FFFF & (unsigned long)(a)))
#define getsegment(a)	((unsigned)((0xFFFF0000 & (unsigned long)(a)) >> 16))
#define ptl(a)		(unsigned long) ( getoffset(a) +  (0x000FFFFF & (((unsigned long)getsegment(a))<<4) ) )
#define ltp(a)		(unsigned long) ( ((ulong)(a) % 16) + (((unsigned long)(a) / 16) << 16) )
#define	 setvec(_iv,_a)		*((char **)((unsigned)_iv*4)) = (char *)_a
#define  getvec(_iv)		(*((char **)((unsigned)_iv*4)))

#define SIRQS_ISOLATED 1
typedef enum SCR_COLORS
{
        SCR_White = 0x7,
        SCR_Black = 0x0,
        SCR_Red = 0x4,
        SCR_Green = 0x2,
        SCR_Yellow = 0x6,
        SCR_Cyan = 0x3,
        SCR_Purple = 0x5,
        SCR_Blue = 0x1
} SCR_COLORS;

#define ds_store (4*0x28) /* In place of DOS IDLE loop interrupt */

#define IRQVEC_FOR_SER_P1 0xc
#define IRQVEC_FOR_SER_P2 0xb
/* Variables used in simplified versions of Serial Port drivers */
#define BUFLEN 1024
#define ISEM1 (MAXSEM - 1)
#define OSEM1 (MAXSEM - 2)
#define ISEM2 (MAXSEM - 3)
#define OSEM2 (MAXSEM - 4)
#define MAXCHARS_BEFORE_STOPPING 6

typedef enum BAUD_RATE
{
	B110 = 0x0417,
	B150 = 0x0300,
	B300 = 0x0180,
	B600 = 0x00C0,
	B1200 = 0x0060,
	B2400 = 0x0030,
	B4800 = 0x0018,
	B9600 = 0x000C
} BAUD_RATE;

typedef enum SER_PORT
{
	SER_P1 = 0,
	SER_P2 = 2
} SER_PORT;

struct IrqConf
{
	void (*IrqActions)(void);
	unsigned char IR_Byte;
	unsigned char Chars_Bef_Irq;
};



/* Error_Type: */
/* 0 No error */
/* 1 Read error */
/* 3 Read driver internal buffer full. Char discarded */
/* 2 Write error */
struct QSR_ERROR
{
	unsigned char Error_Type;
	int Error;
};

/* for IR_Byte: For example: Enable interrupts for reading and writing for every incoming/outgoing char: */
/* struct IrqConf *x = malloc(sizeof(struct IrqConf)); */
/* x->IR_Byte = IRQ_When_Char_Ready | IRQ_When_THR_Empty; */
/* x->Chars_Bef_Irq = ch1; */
#define IRQ_When_Char_Ready 0x1
#define IRQ_When_THR_Empty 0x2

/* For chars_bef_irq */
#define ch1 0x00
#define ch4 0x40
#define ch8 0x80
#define ch14 0xC0
